#!/usr/bin/python
'''
Parses API for C++ from Scriptable API documentation.
'''

import re

readme_path = 'docs/scripting-api.rst'
output_path = 'src/gui/commandcompleterdocumentation.h'

header = '''// Generated by "utils/script_docs_to_cpp.py" from "%s".
template <typename AddDocumentationCallback>
void addDocumentation(AddDocumentationCallback addDocumentation)
{
'''

footer = '}'

# Regex for function/variable/type name in documentation
re_title = re.compile(r'''
  (?:
    ^\.\.\s*js:function::\s*
    (?P<function_api>
      # function return value
      .*?
      # function name
      (?P<function_name>\w+)
      # arguments
      \(.*
    )

    |

    ^\.\.\s*js:data::\s*
    # variable name
    (?P<variable_name>\w+)
    \s*
    # followed by opening parenthesis
    (?P<variable_api>\(.*)

    |

    ^\.\.\s*js:class::\s*
    # type name
    (?P<type_name>\w+)$
  )
  ''', re.VERBOSE)

def main():
    with open(output_path, mode='w', encoding='utf-8') as output_file:
        output_file.write((header % readme_path) + '\n')

        with open(readme_path, mode='r', encoding='utf-8') as readme_file:
            match = None
            for line in readme_file:
                line = line.strip().replace('``', '`')
                if line:
                    if match:
                        name = match.group('function_name') or match.group('variable_name') or match.group('type_name')
                        api = match.group('function_api') or match.group('variable_api') or name
                        description = line.strip()
                        output = '    addDocumentation("{}", "{}", "{}");\n'\
                            .format(name, api, description)
                        output_file.write(output)
                        match = None
                    else:
                        match = re.match(re_title, line)

        output_file.write(footer + '\n')

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
One time script to fix window title translation strings in CopyQ.

Removes unneeded application from window titles from translation files.

The application name is added to the title by using
QGuiApplication::setApplicationDisplayName().
"""
from glob import glob
import re

import xml.etree.ElementTree as ET


def main():
    prefix = re.compile('^CopyQ\\s*')
    suffix = re.compile('\\s*CopyQ$')
    sources = {
        "CopyQ About",
        "CopyQ Action Dialog",
        "CopyQ Add Commands",
        "CopyQ Clipboard Content",
        "CopyQ Commands",
        "CopyQ Configuration",
        "CopyQ Export Error",
        "CopyQ Import Error",
        "CopyQ Item Content",
        "CopyQ Log",
        "CopyQ New Tab",
        "CopyQ New Shortcut",
        "CopyQ Options for Export",
        "CopyQ Options for Import",
        "CopyQ Process Manager",
        "CopyQ Rename Tab",
        "CopyQ Rename Tab Group",
        "CopyQ Select Icon",
    }

    for fn in glob('translations/*.ts'):
        tree = ET.parse(fn)
        root = tree.getroot()
        for context in root.iter('context'):
            for message in context.iter('message'):
                source = message.find('source')
                if source.text in sources:
                    tr = message.find('translation')
                    if not tr.text:
                        continue

                    source.text = re.sub(prefix, '', source.text)
                    tr.text = re.sub(prefix, '', tr.text)
                    tr.text = re.sub(suffix, '', tr.text)
                    if tr.text[0].islower():
                        tr.text = tr.text[0].upper() + tr.text[1:]
                    tr.set('type', 'unfinished')

        tree.write(fn)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
'''
Updates icon font and header files for CopyQ repository.

First argument is path to upacked Font Awesome archive (https://fontawesome.com/).
'''
import json
import os
import sys

from shutil import copyfile
from textwrap import dedent

from fontTools.merge import Merger
from fontTools.ttLib import TTFont

FONTS = [
    'fa-solid-900.ttf',
    'fa-brands-400.ttf',
]
FONT_FILENAME = 'fontawesome.ttf'

SOLID_STYLE = 'solid'
BRANDS_STYLE = 'brands'


def read_icons(icons_json):
    with open(icons_json, 'r') as icons_file:
        icons_content = icons_file.read()
        return json.loads(icons_content)


def write_header_file_preamble(header_file):
    script = os.path.realpath(__file__)
    script_name = os.path.basename(script)
    comment = (
            f'// This file is generated with "{script_name}"'
            + ' from FontAwesome\'s metadata.\n\n')
    header_file.write(comment)


def write_icon_list_header_file(header_icon_list, icons):
    with open(header_icon_list, 'w') as header_file:
        items = []
        for style in [SOLID_STYLE, BRANDS_STYLE]:
            is_brand = 'true' if style == BRANDS_STYLE else 'false'
            for name, icon in icons.items():
                if style in icon['styles']:
                    code = icon['unicode']
                    search_terms = [icon['label'].lower()] + (icon['search']['terms'] or [])
                    search_terms_list = '|'.join(search_terms)
                    items.append('{0x%s, %s, "%s"}' % (code, is_brand, search_terms_list))

        item_list_content = ',\n'.join(items)
        content = dedent('''\
            struct Icon {
                unsigned int unicode;
                bool isBrand;
                const char *searchTerms;
            };

            constexpr Icon iconList[] = {
            %s
            };
        ''') % item_list_content

        write_header_file_preamble(header_file)
        header_file.write(content)


def write_icons_header_file(header_icons, icons):
    with open(header_icons, 'w') as header_file:
        write_header_file_preamble(header_file)
        header_file.write('#ifndef ICONS_H\n')
        header_file.write('#define ICONS_H\n')
        header_file.write('\n')
        header_file.write('enum IconId {\n')

        for name, icon in icons.items():
            label = name.title().replace('-', '')
            code = icon['unicode']
            header_file.write(f'    Icon{label} = 0x{code},' + '\n')

        header_file.write('};\n')
        header_file.write('\n')
        header_file.write('#endif // ICONS_H\n')


def rename_font_family(path):
    """
    Adds suffix to font family it doesn't conflict with font installed on
    system, which could be in incorrect version.

    See: https://github.com/fonttools/fonttools/blob/master/Snippets/rename-fonts.py
    """

    font = TTFont(path)
    name_table = font['name']

    FAMILY_RELATED_IDS = dict(
        LEGACY_FAMILY=1,
        TRUETYPE_UNIQUE_ID=3,
        FULL_NAME=4,
        POSTSCRIPT_NAME=6,
        PREFERRED_FAMILY=16,
        WWS_FAMILY=21,
    )

    for rec in name_table.names:
        if rec.nameID not in FAMILY_RELATED_IDS.values():
            continue

        name = rec.toUnicode()
        if name.startswith('Font Awesome'):
            rec.string = name + ' (CopyQ)'
        elif name.startswith('FontAwesome'):
            rec.string = name + '(CopyQ)'

        assert rec.toUnicode().endswith('(CopyQ)')

    font.save(path)


def copy_fonts(font_awesome_src, target_font_dir):
    font_dir = os.path.join(font_awesome_src, 'webfonts')
    fonts = [
        os.path.join(font_dir, src_name)
        for src_name in FONTS
    ]
    dest_path = os.path.join(target_font_dir, FONT_FILENAME)

    print(f'Merging fonts: {fonts} -> {dest_path}')
    merger = Merger()
    with merger.merge(fonts) as font:
        font.save(dest_path)

    rename_font_family(dest_path)


def main():
    font_awesome_src = sys.argv[1]

    script = os.path.realpath(__file__)
    utils_dir = os.path.dirname(script)
    project_dir = os.path.dirname(utils_dir)
    src_dir = os.path.join(project_dir, 'src')

    header_icon_list = os.path.join(src_dir, 'gui', 'icon_list.h')
    header_icons = os.path.join(src_dir, 'gui', 'icons.h')

    target_font_dir = os.path.join(src_dir, 'images')
    copy_fonts(font_awesome_src, target_font_dir)

    icons_json = os.path.join(
            font_awesome_src, 'metadata', 'icons.json')
    icons = read_icons(icons_json)

    write_icon_list_header_file(header_icon_list, icons)
    print(f'Header file "{header_icon_list}" updated.')

    write_icons_header_file(header_icons, icons)
    print(f'Header file "{header_icons}" updated.')


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
Create new release and upload files to FossHUB.

USAGE: ./fosshub.py <VERSION> <API_KEY>
"""
import requests
import sys

project_id = '5c1195728c9fe8186f80a14b'
fosshub_new_release_url = 'https://api.fosshub.com/rest/projects/{project_id}/releases/'.format(project_id=project_id)
github_release_url = 'https://github.com/hluk/CopyQ/releases/download/v{version}/{basename}'
files = {
    'copyq-v{version}-setup.exe': 'Windows Installer',
    'copyq-v{version}.zip': 'Windows Portable',
    'CopyQ.dmg': 'macOS',
}

version = sys.argv[1]
api_key = sys.argv[2]

# https://devzone.fosshub.com/dashboard/restApi
data = {
    'version': version,
    'files': [{
        'fileUrl': github_release_url.format(version=version, basename=basename.format(version=version)),
        'type': filetype,
        'version': version
    } for basename, filetype in files.items()],
    'publish': True,
}
headers = {
    'X-Auth-Key': api_key
}

response = requests.post(fosshub_new_release_url, json=data, headers=headers)
if response.status_code != 200:
    raise RuntimeError('Unexpected response: ' + response.text)

print('All OK: ' + response.text)
